import json
import yaml
import os
import sys


from Database import Database

LOGICAL_IDS_NAME = "berye_logical_ids"


'''
So this is where most of the translation difficulty is:

_constraints:_
not_null
unique
primary_key
foreign_key
check (logical expression)
default (value)


_attributes:_
auto_increment
storage {disk | memory | default}
comment string
column_format {fixed | dynamic | default}

what about references?



'''


class Column(object):


    def __init__(self, column_data, database=None, table=None):
        self.table = table
        self.database = database
        self.logical_id = column_data.get("logical_id")

        self.column_name = column_data.get("column_name")
        self.data_type = column_data.get("data_type")

        # this might be covered in the data type
        self.max_length = column_data.get("max_length")

        # this will be covered by the second pass
        self.index = column_data.get("index", False)

        # theoretically this should be covered in the first pass, I think?
        self.primary_key = column_data.get("primary_key", False)


        self.auto_increment = column_data.get("auto_increment", False)
        self.comment = column_data.get("comment")
        self.unique = column_data.get("unique", False)
        self.not_null = column_data.get("not_null", False)
        self.check = column_data.get("check")
        self.default = column_data.get("default")
        self.references = column_data.get("references", [])

        # Handling Foreign Keys
        self.foreign_key = column_data.get("foreign_key", False)
        self.reference_table = column_data.get("reference_table")
        self.reference_column = column_data.get("reference_column", "id")

        self.previous_column = None
        self.future_column = None

        self.unchanged = True
        self.changes = []

        if self.reference_table:
            key_name = "%s_%s" % (self.reference_table, self.reference_column)
            self.foreign_key_name = column_data.get("foreign_key_name", key_name)
        else:
            self.foreign_key_name = column_data.get("foreign_key_name")

        return None


    def sql(self):
        sql = ("%s %s" % (self.column_name, self.data_type))

        if self.not_null:
            sql += " NOT NULL"

        if self.default is not None:
            sql += (" DEFAULT %s" % self.default)

        if self.auto_increment:
            sql += " AUTO_INCREMENT"

        if self.primary_key:
            sql += " PRIMARY KEY"

        if self.comment is not None:
            sql += (" COMMENT '%s'" % self.comment)

        if self.unique:
            sql += " UNIQUE"

        if self.check is not None:
            sql += (" CHECK (%s %s)" % (self.column_name, self.check))

        return sql


    def createForeignKeySql(self):
        if not self.foreign_key:
            return None
        sql = "ADD CONSTRAINT %s FOREIGN KEY (%s) REFERENCES %s (%s)" % (
            self.foreign_key_name,
            self.column_name,
            self.reference_table,
            self.reference_column
        )
        return sql


    def dropForeignKeySql(self):
        if not self.foreign_key:
            return None
        return ""


    def findPreviousColumn(self, database_columns):
        previous_columns = filter(lambda x: x.logical_id == self.logical_id, database_columns)
        print "---"
        print "FINDING PREVIOUS COLUMN"
        print "%s Database Columns To Search" % str(len(database_columns))
        print "---"
        if len(previous_columns) > 0:
            print "FOUND A PREVIOUS COLUMN"
            print previous_columns[0].column_name
            self.previous_column = previous_columns[0]
            self.previous_column.future_column = self
            print "***"
            return self.previous_column
        return None


    def matchesPreviousColumn(self):
        attributes_to_match = ("data_type", "column_name", "index",
            "primary_key", "auto_increment", "unique", "not_null", "default")
        # what are the things that we are going to want to see in here?
        differences = []

        for attribute in attributes_to_match:
            if getattr(self, attribute) != getattr(self.previous_column, attribute):
                differences.append(attribute)

        # TODO : this is code smell, find a better way to put this.
        foreign_key_attributes_to_match = ("foreign_key", "reference_table", "reference_column")
        for attribute in foreign_key_attributes_to_match:
            if getattr(self, attribute) != getattr(self.previous_column, attribute):
                if "foreign_key" not in differences:
                    differences.append("foreign_key")

        self.changes = differences
        if len(self.changes) > 0:
            self.unchanged = False

        return self.unchanged


    def changeColumn(self):
        print "About to change the column: %s" % self.column_name
        print "Changes to make: %s" % str(self.changes)

        # remove foreign key if necessary
        if "foreign_key" in self.changes:
            self.removeForeignKey()

        # remove the index if necessary
        if "index" in self.changes and self.previous_column.index == True:
            self.removeIndex()

        modify_triggers = ("column_name", "data_type", "not_null", "default", "auto_increment", "primary_key", "unique")
        if any((x in self.changes for x in modify_triggers)):
            # NB : This will fail if it's an incompatible data type, instead of overriding
            # at the moment we want this behaviour but that may change in the future
            # if we have a FORCE command added. It's just that this was the most
            # logical way to not lose the data
            self.modifyColumn()

        # add the index if appropriate
        if "index" in self.changes and self.index == True:
            self.addIndex()

        return None


    def removeForeignKey(self):
        sqlquery = """
            ALTER TABLE %s DROP FOREIGN KEY %s;
        """ % (self.table.name, self.previous_column.foreign_key_name)
        return self.database.query(sqlquery)


    def removeIndex(self):
        sqlquery = """
            DROP INDEX %s ON %s;
        """ % (self.column_name, self.table.name)
        return self.database.query(sqlquery)


    def modifyColumn(self):
        if "column_name" in self.changes:
            # change
            modify_string = "CHANGE %s %s" % (self.previous_column.column_name, self.sql())
        else:
            # modify
            modify_string = "MODIFY %s" % self.sql()
        sqlquery = """
            ALTER TABLE %s %s;
        """ % (
            self.table.name,
            modify_string
        )
        return self.database.query(sqlquery)


    def addIndex(self):
        sqlquery = """
            ADD INDEX %s ON %s;
        """ % (self.column_name, self.table.name)
        return self.database.query(sqlquery)


    def addForeignKey(self):
        return self.database.query(self.createForeignKeySql())


    def dropString(self):
        return "DROP COLUMN %s" % self.column_name


    def addString(self):
        return "ADD COLUMN %s" % self.sql()


    def updateLogicalId(self):
        sqlquery = """
            SELECT id FROM %s WHERE table_name = '%s' AND logical_id = '%s'
        """ % (LOGICAL_IDS_NAME, self.table.name, self.logical_id)
        rows = self.database.query(sqlquery)
        if len(rows) == 0:
            # create it from scratch
            insertquery = """
                INSERT INTO %s (table_name, column_name, logical_id) VALUES
                ('%s', '%s', '%s');
            """ % (
                LOGICAL_IDS_NAME,
                self.table.name,
                self.column_name,
                self.logical_id
            )
            print insertquery
            self.database.query(insertquery)
        else:
            # this needs to be an update
            updatequery = """
                UPDATE %s SET column_name = '%s' WHERE id = %s;
            """ % (
                LOGICAL_IDS_NAME,
                self.column_name,
                rows[0]['id']
            )
            print updatequery
            self.database.query(updatequery)
        return None


    def retrieveLogicalId(self):
        if self.logical_id is not None and self.logical_id != "":
            sqlquery = """
                SELECT id, column_name FROM %s WHERE table_name = '%s' AND logical_id = '%s';
            """ % (
                LOGICAL_IDS_NAME,
                self.table.name,
                self.logical_id
            )
            rows = self.database.query(sqlquery)
            if len(rows) > 0:
                return rows[0]

        # if it couldn't be found that way then try finding it via table and column
        backupquery = """
            SELECT id, column_name, logical_id FROM %s WHERE table_name = '%s' AND column_name = '%s';
        """ % (
            LOGICAL_IDS_NAME,
            self.table.name,
            self.column_name
        )
        rows = self.database.query(backupquery)
        if len(rows) > 0:
            # in here we should be setting the logical_id
            self.logical_id = rows[0]['logical_id']
            return rows[0]
        return None


    def exists(self):
        return None



import os
import json
import yaml
import sys
import re

class Config(object):

    def __init__(self, host=None, port=None, database=None, username=None, password=None):
        self.file_name = self.configFileName()
        self.config_dict = None

        if self.file_name is not None:
            # is it json or yaml?
            self.format = "yaml"
            if ".json" in self.file_name:
                self.format = "json"

            self.configDict()
        else:
            self.config_dict = {}

        # look for the mains
        if host is None:
            self.host = self.config_dict.get("host", "")
        else:
            self.host = host

        if port is None:
            self.port = self.config_dict.get("port", 3306)
        else:
            self.port = port

        if database is None:
            self.database = self.config_dict.get("database", "")
        else:
            self.database = database

        if username is None:
            self.username = self.config_dict.get("username", "")
        else:
            self.username = username

        if password is None:
            self.password = self.config_dict.get("password", "")
        else:
            self.password = password

        return None

    def configFileName(self):
        # we need to get the config in here from the appropriate file
        current_directory = os.getcwd()
        try:
            schema_files = os.listdir('%s/schema' % current_directory)
        except OSError:
            return None
        config_files = filter(lambda x: x.split(".")[0] == "berye_config", schema_files)
        if len(config_files) == 0:
            print "No Config Available"
            # TODO : Perhaps we're better off throwing an error here becuase
            # without the config we're going to have no way of proceeding.
            return None
        config_file_name = config_files[0]
        return config_file_name


    def configDict(self):

        if self.config_dict is not None:
            return self.config_dict

        if self.file_name is None:
            self.config_dict = {}
            return self.config_dict

        current_directory = os.getcwd()

        # get the file and convert it to a native dict
        with open("%s/schema/%s" % (current_directory, self.file_name)) as f:
            if self.format == "yaml":
                self.config_dict = yaml.load(f)
            else:
                self.config_dict = json.load(f)

        # replace any variables
        self.parseConfigVariable(self.config_dict)


        return self.config_dict


    '''
    Config Variables will be able to come in a number of forms
    but for the moment we're just going to handle the ability to put in
    ENV variables so that we can store database passwords etc in them
    '''
    def parseConfigVariable(self, o):
        if isinstance(o, dict):
            for key in o.keys():
                o[key] = self.parseConfigVariable(o[key])
        elif isinstance(o, list) or isinstance(o, tuple):
            for item in o:
                item = self.parseConfigVariable(item)
        elif isinstance(o, str):
            o = self.parseConfigString(o)
        elif isinstance(o, unicode):
            try:
                item = str(o)
                o = self.parseConfigString(item)
            except Exception as exception:
                print "There was an error trying to parse %u: %s" % (o, str(exception))
                return None
        return o


    def parseConfigString(self, o):
        env_start = '\${'
        env_end = '}'
        env_var_refs = re.findall(env_start + '(.*?)' + env_end, o, re.DOTALL)
        for ref in env_var_refs:
            o = o.replace("${" + ref + env_end, self.parseEnvVariable(ref))
        return o


    def parseEnvVariable(self, variable_name):
        variable_name = variable_name.upper()
        return os.environ.get(variable_name, "")



import json
import yaml
import os
import sys

from Database import Database
from Column import Column

schema_extensions = (".json", ".yaml", ".yml")
LOGICAL_IDS_NAME = "berye_logical_ids"


'''
There are also some table options in here that I need to think about
Auto Increment
Average Row length
Default Character Set
Checksum
Collate
Comment
Compression
Connection
Data Directory
Delay Key write
Encryption
Engine
Insert Method
Key Block Size
Max Rows
Min Rows
Pack Keys
Password
Row Format
Stats Auto Recalc
Stats Persistent
Stats Sample Pages
Tablespace
Union

When creating we need to make sure that we're doing it in order:

make sure that all of the columns are there first and foremost
then create the indexes
then we will add the foreign keys afterwards, this is probably the best way of
doing it.

'''


class Table(object):


    def __init__(self, table_name, database=None):
        self.name = table_name
        self.columns = []
        self.database_columns = []
        self.references = []
        self.schema_file = None
        self.file_format = None
        self.schema_dict = {}
        if database is None:
            self.database = Database()
        else:
            self.database = database
        return None


    def exists(self):
        rows = self.database.query(
            "SELECT table_name FROM information_schema.tables;"
        )
        print rows
        tables = map(lambda x: x["table_name"], rows)
        return self.name in tables


    def parseSchema(self):
        self.getSchemaFile()
        if self.schema_file is None:
            return None

        with open(self.schema_file) as f:
            if self.file_format == "json":
                self.schema_dict = json.load(f)
            else:
                self.schema_dict = yaml.load(f)

        self.object_name = self.schema_dict.get("object_name", None)
        self.raw_column_data = self.schema_dict.get("columns", [])
        self.references = self.schema_dict.get("references", [])
        self.columns = []
        for column_data in self.raw_column_data:
            self.columns.append(Column(column_data, database=self.database, table=self))
        # handling for primary key
        primary_key_columns = filter(lambda x: x.primary_key == True, self.columns)
        if len(primary_key_columns) == 1:
            return None
        elif len(primary_key_columns) > 1:
            # There are too many primary keys, throw an exception
            raise Exception("Too Many Primary Keys", "%s has %s primary keys defined" % (self.name, str(len(primary_key_columns))))
        else:
            # There are no primary key columns

            # if this is the case then first we need to check if there is a
            # column called id or {table_name}_id and we should just assume
            # that this was the intended id, otherwise we create one
            likely_id_fields = filter(
                lambda x: (x.column_name == "id" or x.column_name == self.name + "_id"),
                self.columns
            )
            if len(likely_id_fields) > 0:
                likely_id_fields.sort(key=len)
                likely_id_fields[0].primary_key = True
                likely_id_fields[0].not_null = True
                likely_id_fields[0].auto_increment = True
            else:
                self.columns.append(Column(
                    {
                        "column_name" : "id",
                        "logical_id" : "id",
                        "auto_increment" : True,
                        "data_type" : "INT",
                        "primary_key" : True,
                        "not_null" : True
                    },
                    database=self.database,
                    table=self
                ))
        return None


    def getSchemaFile(self):
        if self.schema_file is not None:
            return self.schema_file
        current_directory = os.getcwd()
        schema_files = os.listdir("%s/schema/" % current_directory)
        possible_schema_file_names = map(lambda x: self.name + x, schema_extensions)
        matching_files = filter(lambda x: x in possible_schema_file_names, schema_files)
        if len(matching_files) == 0:
            return None
        self.schema_file = "%s/schema/%s" % (current_directory, matching_files[0])
        if ".json" in self.schema_file:
            self.file_format = "json"
        else:
            self.file_format = "yaml"
        return self.schema_file


    def foreignKeysExist(self):
        return (len(self.missingForeignKeys()) == 0 and len(self.superfluousForeignKeys()) == 0)


    def missingForeignKeys(self):
        keys = self.getDatabaseForeignKeys()
        schema_keys = filter(lambda x: x.foreign_key == True, self.columns)
        schema_keys = map(lambda x: x.foreign_key_name, schema_keys)
        return filter(lambda x: x not in keys, schema_keys)


    def superfluousForeignKeys(self):
        keys = self.getDatabaseForeignKeys()
        schema_keys = filter(lambda x: x.foreign_key == True, self.columns)
        schema_keys = map(lambda x: x.foreign_key_name, schema_keys)
        return filter(lambda x: x not in schema_keys, keys)


    def getDatabaseForeignKeys(self):
        self.database.query("USE INFORMATION_SCHEMA;")
        query = """
        SELECT CONSTRAINT_NAME FROM KEY_COLUMN_USAGE
        WHERE TABLE_SCHEMA = '%s'
        AND TABLE_NAME = '%s'
        AND REFERENCED_COLUMN_NAME IS NOT NULL
        """ % (self.database.config.database, self.name)
        rows = self.database.query(query)

        # TODO : remove this once we know that the new cursor style is working
        print rows
        keys = map(lambda x: x['CONSTRAINT_NAME'], rows)
        return keys


    '''

    Migrate the core of the table

    '''

    def migrate(self):
        if not self.exists():
            self.create()
            return True

        self.update()

        return False

    def create(self):
        if len(self.columns) == 0:
            self.parseSchema()
        if self.database.logicalTableExists() == False:
            self.database.createLogicalIdTable()
        index_columns = filter(lambda x: x.index == True, self.columns)
        sql = "CREATE TABLE %s (%s" % (
            self.name,
            ", ".join(map(lambda x: x.sql(), self.columns))
        )
        if len(index_columns) > 0:
            sql += ", %s" % ", ".join(map(lambda x: "INDEX(%s)" % x.column_name, index_columns))
        sql += ");"
        print sql
        rows = self.database.query(sql)
        print rows
        self.createLogicalIds()
        return sql


    def createLogicalIds(self):
        return map(lambda x: x.updateLogicalId(), self.columns)


    def update(self):
        # work out what's there that's the same and what's there that isn't
        # TODO : we need to work out how the update is going to work, we
        # should have some kind of thing previously
        if len(self.columns) == 0:
            self.parseSchema()
        self.databaseDefinition()
        map(lambda x: x.findPreviousColumn(self.database_columns), self.columns)

        unchanged_columns = filter(lambda x: x.previous_column is not None, self.columns)

        new_columns = filter(lambda x: x.previous_column is None, self.columns)
        removeable_columns = filter(lambda x: x.future_column is None, self.database_columns)
        changed_columns = filter(lambda x: x.unchanged == False, unchanged_columns)
        unchanged_columns = filter(lambda x: x.unchanged == True, unchanged_columns)

        # remove columns
        self.removeColumns(removeable_columns)

        # add columns
        self.addColumns(new_columns)

        # change columns ??
        self.changeColumns(changed_columns)

        self.createLogicalIds()

        return None


    def removeColumns(self, removeable_columns):
        if len(removeable_columns) == 0:
            return None
        drop_string = ", ".join(map(lambda x: x.dropString(), removeable_columns))
        return self.alterTable(drop_string)


    def addColumns(self, new_columns):
        if len(new_columns) == 0:
            return None
        add_string = ", ".join(map(lambda x: x.addString(), new_columns))
        return self.alterTable(add_string)


    def changeColumns(self, changed_columns):
        return map(lambda x: x.changeColumn(), changed_columns)


    def addNewForeignKeys(self):
        columns_to_migrate = filter(lambda x: (x.unchanged == False and "foreign_key" in x.changes), self.columns)
        return map(lambda x: x.addForeignKey(), columns_to_migrate)


    def alterTable(self, alter_string):
        sqlquery = """
        ALTER TABLE %s %s;
        """ % (self.name, alter_string)
        return self.database.query(sqlquery)

    '''

    Migrate the foreign keys on the table

    '''


    def createForeignKeys(self):

        foreign_keys = filter(lambda x: x.foreign_key == True, self.columns)
        # TODO : We are going to need a thing in here that figures out whether
        # or not the key needs to be created.
        if len(foreign_keys) == 0:
            return None

        key_sql = map(lambda x: x.createForeignKeySql(), foreign_keys)
        sql = "ALTER TABLE %s %s;" % (
            self.name,
            " ".join(key_sql)
        )
        print sql
        rows = self.database.query(sql)
        print rows
        return sql


    # TODO : move these around later
    def databaseIndexes(self):
        # get the indexes
        indexquery = "SHOW INDEX FROM %s" % self.name
        rows = self.database.query(indexquery)
        rows = filter(lambda x: x['Key_name'] != "PRIMARY", rows)
        return rows


    def databaseForeignKeys(self):
        keysquery = """
        SELECT
          TABLE_NAME,COLUMN_NAME,CONSTRAINT_NAME,
          REFERENCED_TABLE_NAME,REFERENCED_COLUMN_NAME
        FROM
          INFORMATION_SCHEMA.KEY_COLUMN_USAGE
        WHERE
          TABLE_NAME = '%s';
        """ % self.name
        rows = self.database.query(keysquery)
        return rows


    def databaseConstraints(self):
        constraintquery = """
        SELECT *
        FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS
        WHERE CONSTRAINT_SCHEMA = '%s'
        AND TABLE_NAME = '%s'
        AND CONSTRAINT_TYPE = 'UNIQUE'
        """ % (self.database.config.database, self.name)
        rows = self.database.query(constraintquery)
        return rows


    def databaseDefinition(self):
        if len(self.database_columns) > 0:
            return self.database_columns

        indexes = self.databaseIndexes()
        foreign_keys = self.databaseForeignKeys()
        constraints = self.databaseConstraints()

        # get the columns
        rows = self.database.query("SHOW FULL COLUMNS FROM %s" % self.name)
        for row in rows:
            potential_indexes = filter(lambda x: x.get("COLUMN_NAME", "") == row['Field'], indexes)
            potential_keys = filter(lambda x: x.get("Column_name", "") == row['Field'], foreign_keys)
            potential_constraints = filter(lambda x: x.get("CONSTRAINT_NAME", "") == row['Field'], constraints)

            # we need to get the logical id (this feels like it should be a batch thing)

            column_def = {
                "column_name" : row['Field'],
                "data_type" : row['Type'].upper(),
                "index" : (len(potential_indexes) > 0), # can't find this yet either
                "primary_key" : (row['Key'] == "PRI"),
                "auto_increment" : ("auto_increment" in row['Extra']),
                "comment" : row['Comment'],
                "unique" : (len(potential_constraints) > 0), # can't see this yet either
                "not_null" : (row['Null'] != "NO"),
                "default" : row['Default'],
                "foreign_key" : (len(potential_keys))
            }
            # took the logical_id out of the def because I am handling it in the
            # model a couple of lines down
            if len(potential_keys) > 0:
                column_def['reference_table'] = potential_keys[0]['REFERENCED_TABLE_NAME']
                column_def['reference_column'] = potential_keys[0]['REFERENCED_COLUMN_NAME']
                column_def['foreign_key_name'] = potential_keys[0]['CONSTRAINT_NAME']

            col = Column(column_def, database=self.database, table=self)
            col.retrieveLogicalId()

            self.database_columns.append(col)
        print rows
        print len(rows)
        print len(self.database_columns)

        return self.database_columns


    def initFromDatabase(self):
        return None


    def drop(self):
        dropquery = "DROP TABLE %s;" % self.name
        self.database.query(dropquery)
        return None



import os
import sys
import yaml
import json


import mysql.connector
from Config import Config


LOGICAL_IDS_NAME = "berye_logical_ids"

class Database(object):

    def __init__(self, database_type="mysql"):
        self.database_type = database_type
        self.config = None
        self.connection = None
        self.super_connection = None
        self.tables = []
        return None


    def configure(self, host=None, port=None, database=None, username=None, password=None):
        if self.config is not None:
            return self.config
        self.config = Config(host=host, port=port, database=database, username=username, password=password)
        return self.config


    def exists(self):
        self.configure()
        self.connectToSuper()
        cursor = self.super_connection.cursor()
        cursor.execute("SHOW DATABASES;")
        rows = []
        for row in cursor:
            rows.append(row)
        print rows
        databases = map(lambda x: x[0], rows)
        return self.config.database in databases


    def create(self):
        self.configure()
        self.connectToSuper()
        create_cursor = self.super_connection.cursor()
        create_cursor.execute("CREATE DATABASE %s" % self.config.database)
        self.super_connection.commit()
        rows = []
        for row in create_cursor:
            rows.append(row)
        self.createLogicalIdTable()
        return rows


    def logicalTableExists(self):
        return (LOGICAL_IDS_NAME in self.tableNames())


    def tableNames(self):
        querystring = "SHOW TABLES;"
        rows = self.query(querystring)
        table_names = map(lambda x: x.get("Tables_in_" + self.config.database), rows)
        return table_names


    def createLogicalIdTable(self):
        querystring = "CREATE TABLE " + LOGICAL_IDS_NAME + " "
        querystring += """
            (
            table_name VARCHAR(255),
            column_name VARCHAR(255),
            logical_id VARCHAR(255),
            id INT NOT NULL AUTO_INCREMENT UNIQUE PRIMARY KEY,
            INDEX (id),
            INDEX (table_name),
            INDEX (column_name),
            INDEX (logical_id)
            );
        """
        print "Query"
        print querystring
        rows = self.query(querystring)
        return rows


    def migrate(self):
        # TODO : THIS IS AN UGLY HACK, FIX THIS
        from berye.Table import Table
        if not self.exists():
            self.create()
        # get all of the schema files
        current_directory = os.getcwd()
        schema_files = os.listdir("%s/schema/" % current_directory)
        schema_files = filter(lambda x: "berye_config" not in x, schema_files)
        table_names = map(lambda x: x.split(".")[0], schema_files)
        for table_name in table_names:
            if table_name not in self.tables:
                self.tables.append(Table(table_name, database=self))

        existing_tables = filter(lambda x: x.exists(), self.tables)
        new_tables = filter(lambda x: x.exists() == False, self.tables)

        # migrate all of the tables
        for table in self.tables:
            table.migrate()

        # migrate all of the foreign keys
        for table in new_tables:
            table.createForeignKeys()
        for table in existing_tables:
            table.addNewForeignKeys()

        # find any tables that need to be dropped
        table_names.append(LOGICAL_IDS_NAME)
        database_table_names = self.tableNames()
        superfluous_tables = filter(lambda x: x not in table_names, database_table_names)
        for table_name in superfluous_tables:
            dropquery = "DROP TABLE %s;" % table_name
            print dropquery
            self.query(dropquery)

        return None


    def connect(self):
        if self.connection is not None and self.connection.is_connected():
            return self.connection
        self.configure()
        if self.database_type == "mysql":
            self.connection = mysql.connector.connect(
                user=self.config.username,
                password=self.config.password,
                host=self.config.host,
                database=self.config.database,
                port=self.config.port
            )
        else:
            print "%s not currently supported" % self.database_type
        return self.connection


    def connectToSuper(self):
        if self.super_connection is not None:
            return self.super_connection
        self.configure()
        if self.database_type == "mysql":
            self.super_connection = mysql.connector.connect(
                user=self.config.username,
                password=self.config.password,
                host=self.config.host,
                port=self.config.port
            )
        else:
            print "%s not currently supported" % self.database_type
        return self.super_connection


    def query(self, query):
        print "Query:: %s" % query
        self.connect()
        cursor = self.connection.cursor(buffered=True, dictionary=True)
        cursor.execute(query)

        self.connection.commit()
        rows = []
        for row in cursor:
            rows.append(row)
        return rows


    def drop(self):
        self.configure()
        self.connectToSuper()
        drop_cursor = self.super_connection.cursor()
        drop_cursor.execute("DROP DATABASE %s" % self.config.database)
        self.super_connection.commit()
        rows = []
        for row in drop_cursor:
            rows.append(row)
        return rows



import json
import os
import sys

from Database import Database

DEFAULT_DATABASE_TYPE = "mysql"

class Berye(object):

    def __init__(self, database_type=DEFAULT_DATABASE_TYPE):
        self.database_type = database_type
        self.database = Database(database_type)
        return None

    def create(self):
        self.database.create()
        return None

    def migrate(self):
        # check for the database, if it's not there create it
        database = Database()
        if not database.exists():
            database.create()

        # for each table that we find, check if it exists

        # if it doesn't exist, create it

        # if it does exist, check if the schema matches

        # if the schema doesn't match, update it

        return None

    def query(self, query):
        return None

    def drop(self):
        self.database.drop()
        return None



from setuptools import setup

setup(
    name='berye',
    version='0.1',
    description='A library for making the management of our MySQL schema easier for our lambda services',
    url='https://github.com/tamme-io/berye',
    author='tamme',
    author_email='opensource@tamme.io',
    license='MIT',
    packages=[
        'berye'
    ],
    install_requires=[
    ],
    zip_safe=False,
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
    ]
)
